"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = exports.render = void 0;
const blockattributes_1 = require("./blockattributes");
const DelimitedBlocks = require("./delimitedblocks");
const Io = require("./io");
const LineBlocks = require("./lineblocks");
const Lists = require("./lists");
const Macros = require("./macros");
const Options = require("./options");
const Quotes = require("./quotes");
const Replacements = require("./replacements");
function render(source) {
    const reader = new Io.Reader(source);
    const writer = new Io.Writer();
    while (!reader.eof()) {
        reader.skipBlankLines();
        if (reader.eof())
            break;
        if (LineBlocks.render(reader, writer))
            continue;
        if (Lists.render(reader, writer))
            continue;
        if (DelimitedBlocks.render(reader, writer))
            continue;
        // This code should never be executed (normal paragraphs should match anything).
        Options.panic("no matching delimited block found");
    }
    return writer.toString();
}
exports.render = render;
// Set API to default state.
function init() {
    blockattributes_1.BlockAttributes.init();
    Options.init();
    DelimitedBlocks.init();
    Macros.init();
    Quotes.init();
    Replacements.init();
}
exports.init = init;
//# sourceMappingURL=api.js.map