"use strict";
// deno-lint-ignore-file no-explicit-any
Object.defineProperty(exports, "__esModule", { value: true });
exports.replaceInline = exports.merge = exports.copy = exports.replaceMatch = exports.replaceSpecialChars = exports.escapeRegExp = void 0;
const Macros = require("./macros");
const Options = require("./options");
const Spans = require("./spans");
// http://stackoverflow.com/questions/3561493/is-there-a-regexp-escape-function-in-javascript
function escapeRegExp(s) {
    return s.replace(/[\-\/\\^$*+?.()|\[\]{}]/g, "\\$&");
}
exports.escapeRegExp = escapeRegExp;
function replaceSpecialChars(s) {
    return s.replace(/&/g, "&amp;")
        .replace(/>/g, "&gt;")
        .replace(/</g, "&lt;");
}
exports.replaceSpecialChars = replaceSpecialChars;
// Replace pattern '$1' or '$$1', '$2' or '$$2'... in `replacement` with corresponding match groups
// from `match`. If pattern starts with one '$' character add specials to `expansionOptions`,
// if it starts with two '$' characters add spans to `expansionOptions`.
function replaceMatch(match, replacement, expansionOptions = {}) {
    return replacement.replace(/(\${1,2})(\d)/g, function () {
        // Replace $1, $2 ... with corresponding match groups.
        if (arguments[1] === "$$") {
            expansionOptions.spans = true;
        }
        else {
            expansionOptions.specials = true;
        }
        const i = Number(arguments[2]); // match group number.
        const result = match[i]; // match group text.
        if (result === undefined) {
            Options.errorCallback("undefined replacement group: " + arguments[0]);
            return "";
        }
        return replaceInline(result, expansionOptions);
    });
}
exports.replaceMatch = replaceMatch;
// Shallow object clone.
function copy(source) {
    const result = {};
    for (const key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
            result[key] = source[key];
        }
    }
    return result;
}
exports.copy = copy;
// Copy properties in source object to target object.
function merge(target, source) {
    for (const key in source) {
        target[key] = source[key];
    }
}
exports.merge = merge;
// Replace the inline elements specified in options in text and return the result.
function replaceInline(text, expansionOptions) {
    if (expansionOptions.macros) {
        text = Macros.render(text);
    }
    // Spans also expand special characters.
    if (expansionOptions.spans) {
        text = Spans.render(text);
    }
    else if (expansionOptions.specials) {
        text = replaceSpecialChars(text);
    }
    return text;
}
exports.replaceInline = replaceInline;
//# sourceMappingURL=utils.js.map