export class Reader {
    constructor(text) {
        text = text.replace("\u0000", " "); // Used internally by spans package.
        text = text.replace("\u0001", " "); // Used internally by spans package.
        text = text.replace("\u0002", " "); // Used internally by macros package.
        // Split lines on newline boundaries.
        // http://stackoverflow.com/questions/1155678/javascript-string-newline-character
        // Split is broken on IE8 e.g. 'X\n\nX'.split(/\n/g).length) returns 2 but should return 3.
        this.lines = text.split(/\r\n|\r|\n/g);
        this.pos = 0;
    }
    get cursor() {
        console.assert(!this.eof());
        return this.lines[this.pos];
    }
    set cursor(value) {
        console.assert(!this.eof());
        this.lines[this.pos] = value;
    }
    // Return true if the cursor has advanced over all input lines.
    eof() {
        return this.pos >= this.lines.length;
    }
    // Move cursor to next input line.
    next() {
        if (!this.eof())
            this.pos++;
    }
    // Read to the first line matching the re.
    // Return the array of lines preceding the match plus a line containing
    // the $1 match group (if it exists).
    // If an EOF is encountered return all lines.
    // Exit with the reader pointing to the line containing the matched line.
    readTo(find) {
        const result = [];
        let match = null;
        while (!this.eof()) {
            match = this.cursor.match(find);
            if (match) {
                if (match[1] !== undefined) {
                    result.push(match[1]); // $1
                }
                break;
            }
            result.push(this.cursor);
            this.next();
        }
        return result;
    }
    skipBlankLines() {
        while (!this.eof() && this.cursor.trim() === "") {
            this.next();
        }
    }
}
export class Writer {
    constructor() {
        this.buffer = [];
    }
    write(s) {
        this.buffer.push(s);
    }
    toString() {
        return this.buffer.join("");
    }
}
